@@ -250,11 +250,11 @@ void ddx_get_xi(const void* state, const void* ddx, int ncav, double* xi);
  *  \param ncav    Number of cavity points
  *  \param nbasis  Number of basis functions used by DDX
  *  \param nsph    Number of cavity spheres
- *  \param phi_cav Phi array (ncav, )-shaped array
  *  \param psi     Psi array (nbasis, nsph)-shaped array (in column-major ordering)
+ *  \param phi_cav Phi array (ncav, )-shaped array
  */
 void ddx_cosmo_setup(const void* ddx, void* state, int ncav, int nbasis, int nsph,
-                     const double* phi_cav, const double* psi);
+                     const double* psi, const double* phi_cav);
 
 /** In-place adjust the guess inside the state, getting ready to solve a COSMO problem.
  *  Avoid calling this step if you want to use the currently stored solution as an
@@ -278,6 +278,13 @@ void ddx_cosmo_solve(const void* ddx, void* state, double tol);
  *  \param tol Tolerance up to which the problem is solved */
 void ddx_cosmo_solve_adjoint(const void* ddx, void* state, double tol);
 
+/** Compute COSMO energy (without any scaling by (epsilon - 1) / epsilon
+ *  or similar).
+ *  \param state   DDX state
+ *  \param ddx     DDX model
+ */
+double ddx_cosmo_energy(const void* ddx, void* state);
+
 /** Compute the COSMO force terms.
  *  \param ddx     DDX model
  *  \param state   DDX state
@@ -293,11 +300,11 @@ void ddx_cosmo_solvation_force_terms(const void* ddx, void* state, int nsph,
  *  \param ncav    Number of cavity points
  *  \param nbasis  Number of basis functions used by DDX
  *  \param nsph    Number of cavity spheres
- *  \param phi_cav Phi array (ncav, )-shaped array
  *  \param psi     Psi array (nbasis, nsph)-shaped array (in column-major ordering)
+ *  \param phi_cav Phi array (ncav, )-shaped array
  */
 void ddx_pcm_setup(const void* ddx, void* state, int ncav, int nbasis, int nsph,
-                   const double* phi_cav, const double* psi);
+                   const double* psi, const double* phi_cav);
 
 /** In-place adjust the guess inside the state, getting ready to solve a PCM problem.
  *  Avoid calling this step if you want to use the currently stored solution as an
@@ -321,6 +328,11 @@ void ddx_pcm_solve(const void* ddx, void* state, double tol);
  *  \param tol Tolerance up to which the problem is solved */
 void ddx_pcm_solve_adjoint(const void* ddx, void* state, double tol);
 
+/** Compute PCM energy
+ *  \param state   DDX state
+ *  \param ddx     DDX model */
+double ddx_pcm_energy(const void* ddx, void* state);
+
 /** Compute the PCM force terms.
  *  \param ddx     DDX model
  *  \param state   DDX state
@@ -332,6 +344,51 @@ void ddx_pcm_solvation_force_terms(const void* ddx, void* state, int nsph,
 
 // TODO LPB
 
+/** Setup a LPB problem in the passed state.
+ *  \param ddx     DDX model
+ *  \param state   DDX state
+ *  \param ncav    Number of cavity points
+ *  \param nbasis  Number of basis functions used by DDX
+ *  \param nsph    Number of cavity spheres
+ *  \param psi     Psi array (nbasis, nsph)-shaped array (in column-major ordering)
+ *  \param phi_cav Phi array (ncav, )-shaped array
+ *  \param e_cav   Electric field generated by multipoles: (3, ncav)-shaped array
+ *                 (column major)
+ */
+void ddx_lpb_setup(const void* ddx, void* state, int ncav, int nbasis, int nsph,
+                   const double* psi, const double* phi_cav, const double* e_cav);
+
+/** In-place adjust the guess inside the state, getting ready to solve a LPB problem.
+ *  Avoid calling this step if you want to use the currently stored solution as an
+ *  initial guess. tol is the tolerance for solving a simplified initial-guess problem.
+ *  The same tolerance as for the ddx_lpb_solve call should be chosen. */
+void ddx_lpb_guess(const void* ddx, void* state, double tol);
+
+/** In-place adjust the guess inside the state, getting ready to solve an adjoint LPB
+ *  problem. Avoid calling this step if you want to use the currently stored solution as
+ *  an initial guess. tol is the tolerance for solving a simplified initial-guess problem.
+ *  The same tolerance as for the ddx_lpb_solve call should be chosen. */
+void ddx_lpb_guess_adjoint(const void* ddx, void* state, double tol);
+
+/** Solve the forward LPB problem.
+ *  \param state   DDX state
+ *  \param ddx     DDX model
+ *  \param tol Tolerance up to which the problem is solved */
+void ddx_lpb_solve(const void* ddx, void* state, double tol);
+
+/** Solve the adjoint LPB problem.
+ *  \param state   DDX state
+ *  \param ddx     DDX model
+ *  \param tol Tolerance up to which the problem is solved */
+void ddx_lpb_solve_adjoint(const void* ddx, void* state, double tol);
+
+/** Compute LPB energy
+ *  \param state   DDX state
+ *  \param ddx     DDX model */
+double ddx_lpb_energy(const void* ddx, void* state);
+
+// TODO LPB force terms not yet supported in C and python interface
+
 ///@}
 
 /** \name Multipolar solutes */
